/***********************************************************************
*                                                                      *
* (C) 2007, Frank Lemke, Computer Architecture Group,                  *
* University of Mannheim, Germany                                      *
*                                                                      *
* This program is free software; you can redistribute it and/or modify *
* it under the terms of the GNU General Public License as published by *
* the Free Software Foundation; either version 2 of the License, or    *
* (at your option) any later version.                                  *
*                                                                      *
* This program is distributed in the hope that it will be useful,      *
* but WITHOUT ANY WARRANTY; without even the implied warranty of       *
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        *
* GNU General Public License for more details.                         *
*                                                                      *
* You should have received a copy of the GNU General Public License    *
* along with this program; if not, write to the Free Software          *
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 *
* USA                                                                  *
*                                                                      *
* For informations regarding this file contact                         *
*			      office@mufasa.informatik.uni-mannheim.de *
*                                                                      *
***********************************************************************/

#include <QtGui/QFileDialog>
#include <QtGui/QMessageBox>
#include <QtCore/QString>
#include <fstream>

#include "generationofverilog.h"
#include "../verification/verify.h"
#include "../verification/logicmin.h"
#include "../verification/invertDNF.h"

#include "../corefactory.h"
#include "../core_I.h"
#include "../fsm_I.h"
#include "../load_I.h"
#include "../loadfactory.h"

GenerationOfVerilog::GenerationOfVerilog(string str, string stri, bool gui, bool st, bool sc, bool tc) {
  resetstate = 0;
  reset = 0;
  clock = 0;
  numberofinputs = 0;
  numberofoutputs = 0;
  fsmdata = str;
  optionalpath = stri;
  g=gui;
  Statetrans = st;
  Statecov = sc;
  Transcov = tc;
 
  if(g==true) {
    Verify v(str);
    v.verifyFsm();
    if(v.getVerified()) {
      v.info();
      createVerilog();
    } else {
      v.info();
      if(wanttogenver()) {
	createVerilog();
      }      
    }
  } else {
    cout<<"Not doing verification in command line mode."<<endl;
    Core_I& c = CoreFactory::getCore_I();
    Load_I& l = LoadFactory::getLoad_I();
    l.load(fsmdata);
    createVerilog();
  }
}

bool GenerationOfVerilog::wanttogenver() {
  cout<<"iwashere"<<endl;
  QWidget w;
  int genver = QMessageBox::question(&w, "Generate Verilog", 
				  "Do you still want to generate Verilog?"
				  , QMessageBox::Yes, QMessageBox::Cancel, QMessageBox::NoButton);
  
  if (genver ==  QMessageBox::Yes) {
    return true;
  } else {
    return false;
  }
}

string GenerationOfVerilog::getVerilogFilename() {
  string dirname = fsmdata;
  QString name ;
  string verilogname;

  if(g==true) {
    int len = dirname.length();
    if (len > 0) {
      string s;
      s = dirname;
      do {
      	len--;
      } while ( (s[len] != '/') && (len != 0));
      if (len == 0) {
	dirname = "";
      } else {
      	dirname.assign(s, 0, (len + 1));
      }
    }
    if( optionalpath == "none") {
      QWidget w;
   	 name = QFileDialog::getSaveFileName(&w, "Choose a filename to save Verilog", QString::fromStdString(dirname), "Verilog (*.v)");
    } else {
      QWidget w;
      name = QFileDialog::getSaveFileName(&w, "Choose a filename to save Verilog", 
					  QString::fromStdString(optionalpath), "Verilog (*.v)");
  	}
  } else {
    name=QString::fromStdString(optionalpath);
  } 
  if(!name.isNull()) {
    int len = name.length();
    if(name.at(len - 1) == 'v' && name.at(len - 2) == '.') {
      name.remove(len - 2, len - 1);
    }
    verilogname = name.toStdString() + ".v"; 
  } else {
    verilogname = "";
  }    
  return verilogname;
}


void GenerationOfVerilog::createVerilog() {
  
  /// Data initialization
  cout<<"creating verilog"<<endl;
  Core_I& c = CoreFactory::getCore_I();
  if(c.getFirstFsm()) {
    Fsm_I& f = c.getCurrentFsm();
    string filename = getVerilogFilename();
    if(!filename.empty()){
      const char* const dateiname = filename.c_str();
      ofstream out(dateiname, ios_base::out|ios_base::trunc);
      if(out.is_open()) {
	int maxlen = filename.length();
	int len = maxlen;
	if (len > 0) {
	  string s;
	  s = filename;
	  do {
	    len--;
	  } while ( (s[len] != '/') && (len != 0));
	  if (len == 0) {
	    modulename = "";
	  } else {
	    modulename.assign(s, (len + 1), (maxlen - len - 3));
	  }
	}
	numberofinputs = f.getNumberOfInputs();
	numberofoutputs = f.getNumberOfOutputs();
	resetstate = f.getResetState();

	///errordetection

	if (numberofinputs == 0 || numberofoutputs == 0) {
	  cout << "Error: illegal input or output length" << endl;
	  out.close();
	}else{

	//// Write module header 

	out << "module " << modulename << " ( " ; 
	out << f.getClockName() << ", "; 
	out << f.getResetName() << ", ";
	for(int i = 0; i < numberofinputs; i++) {
	  out << f.getInputName(i) << ", ";
	}
	out << endl;
	for(int i = 0; i < numberofoutputs - 1; i++) {
	  out << f.getOutputName(i) << ", ";
	}
	out << f.getOutputName(numberofoutputs - 1);
	out << " );" << endl;
	out << endl;
	out << "input wire ";
	out << f.getClockName() << ", "; 
	out << f.getResetName() << ", ";
	for(int i = 0; i < numberofinputs - 1; i++) {
	  out << f.getInputName(i) << ", ";
	}
	out << f.getInputName(numberofinputs - 1) << ";" << endl;
	out << "output wire ";
 	for(int i = 0; i < numberofoutputs - 1; i++) {
	  out << f.getOutputName(i) << ", ";
	}
	out << f.getOutputName(numberofoutputs - 1)  << ";" << endl << endl;
	if(f.getFirstState()) {
	  do {
	    out << "parameter " << f.getStateName() << " = " << numberofoutputs << "'b" << f.getStateOutput() << ";" << endl;
	      } while(f.getNextState());
	}
	out << endl << "reg [" << (numberofoutputs - 1) << ":0] current_state, next_state;" << endl; 
	out << "assign {";
	for(int i = 0; i < numberofoutputs - 1; i++) {
	  out << f.getOutputName(i) << ", ";
	}
	out << f.getOutputName(numberofoutputs - 1);
	out << "} = current_state;" << endl << endl;
	out << "wire [" << (numberofinputs - 1) << ":0] inputvector;" << endl;
	out << "assign inputvector = {";
	for(int i = 0; i < numberofinputs - 1; i++) {
	  out << f.getInputName(i) << ", ";
	}
	out << f.getInputName(numberofinputs - 1);	      
	out << "};" << endl << endl << endl;

	//// Write combinational logic always-block

	out << "always @(*) begin" << endl;
	out << "  casex({inputvector, current_state})" << endl;
	if(f.getFirstState()) {
	  int stateid;
	  string statename;
	  do {
	    stateid = f.getStateID();
	    statename = f.getStateName();
	    if(f.getFirstTrans()) {
	      do {
		if(f.getTransStart() == stateid) {
		  int transid = f.getTransID();
		  f.getStatebyID(f.getTransEnd());
		  if(f.getTransDefault()) {
		    out << "    {" << numberofinputs << "'b";
		    vector<string> input;
		    bool notfound = true; 
		    if(f.getFirstTrans()) {
		      do {
			if(f.getTransStart() == stateid && !f.getTransDefault()) {
			  if(f.getTransFirstCondition()) {
			    notfound = false;
			    do {
			      string condition = f.getTransConditionValue();
			      string result = "";
			      char X;
			      for(int i = 0; i < numberofinputs; i++) { 
				X = condition[i];
				if( X == 'x' || X == 'X') {
				  result = result + '-';
				} else {
				  result = result + X;
				}
			      }
			      input.push_back(result);       
			    } while (f.getTransNextCondition());
			  }
			}
		      } while (f.getNextTrans());
		    }
		    if(f.getFirstHypertrans()) {
		      do {
			if(f.getHypertransFirstCondition()) {
			  notfound = false;
			  do {
			    string condition = f.getHypertransConditionValue();
			    string result = "";
			    char X;
			    for(int i = 0; i < numberofinputs; i++) { 
			      X = condition[i];
			      if( X == 'x' || X == 'X') {
				result = result + '-';
			      } else {
				result = result + X;
			    }
			    }
			    input.push_back(result);  
			  } while(f.getHypertransNextCondition());
			}
		      } while (f.getNextHypertrans());
		    }
		    if(input.size()>0) {
		      //the old invert functionality did not produce correct results under rare circumstances
		      //pair<string, string> result;
		      //logicMin(input,result);
		      InvertDNF INV;
		      string defaultvalue = INV.invert(input);
		      int maxlen = defaultvalue.length();
		      int len = maxlen;	 
		      if(len > 0) {
			while (len > numberofinputs) {
			  string s = defaultvalue;
			  do {
			    len--;
			  } while ( (s[len] != '+') && (len != 0));
			  s.assign(s, (len + 1), (maxlen - len - 1));
			  defaultvalue.assign(defaultvalue, 0, len);
			  char X;
			  for(int i = 0; i < numberofinputs; i++) { 
			    X = s[i];
			    if( X == '-' || X == 'X') {
			      out << "x";
			    } else {
			      out << X;
			    }
			  }
			  out << ", " << statename << "}:   next_state = " << f.getStateName() << ";" << endl;
			  out << "    {" << numberofinputs << "'b";
			}
			char X;
			for(int i = 0; i < numberofinputs; i++) { 
			  X = defaultvalue[i];
			  if( X == '-' || X == 'X') {
			    out << "x";
			  } else {
			    out << X;
			  }
			}
			out << ", " << statename << "}:   next_state = " << f.getStateName() << ";" << endl;
		      }		      
		    } 
		    if(notfound) {
		      string invec = "";
		      for(int i = 0; i < numberofinputs; i++) {
			invec = invec + "x";
		      }
		      out << invec << ", " << statename << "}:   next_state = " << f.getStateName() << ";" << endl;
		    }
		    f.getTransbyID(transid);
		  } else {
		    if(f.getTransFirstCondition()) {
		      string condition;
		      do {
			out << "    {" << numberofinputs << "'b";
			condition = f.getTransConditionValue();
			char X;
			for(int i = 0; i < numberofinputs; i++) { 
			  X = condition[i];
			  if( X == '-' || X == 'X') {
			    out << "x";
			  } else {
			    out << X;
			  }
			}
			out << ", " << statename << "}:   next_state = " << f.getStateName() << ";" << endl;
		      } while (f.getTransNextCondition());
		    }
		  }
		}
	      } while (f.getNextTrans());
	    } 
	    f.getStatebyID(stateid);
	  } while (f.getNextState());
	}
	if(f.getFirstHypertrans()) {
	  do {
	    if(f.getHypertransFirstCondition()) {
	      string condition;
	      do { 
		out << "    {" << numberofinputs << "'b";
		condition = f.getHypertransConditionValue();
		char X;
		for(int i = 0; i < numberofinputs; i++) { 
		  X = condition[i];
		  if( X == '-' || X == 'X') {
		    out << "x";
		  } else {
		    out << X;
		  } 
		  }
		out << ", " << numberofoutputs << "'b";
		for(int i = 0; i < numberofoutputs; i++) {
		  out << "x";
		}
		f.getStatebyID(f.getHypertransGoal());
		out << "}:   next_state = " << f.getStateName() << ";" << endl;
		} while (f.getHypertransNextCondition());
	      }
	  } while (f.getNextHypertrans());
	}
	out << "    default:  next_state = ";
	if(f.getFirstState()) {
	  bool notfound = true;
	  do {
	    if(resetstate == f.getStateID()) {
	      out << f.getStateName();
		notfound = false;
	    }
	  } while (f.getNextState());
	  if(notfound) {
	    if(f.getFirstState()) {
	      out << f.getStateName();
	    }
	  }
	}
	out << ";" << endl;
	out << "  endcase" << endl;
	out << "end" << endl << endl;

	//// Write always-block => assigns next_state to state

	out << "always @(posedge " << f.getClockName() << " or negedge " << f.getResetName() <<" ) begin" << endl;
       	out << "  if (" << f.getResetName() << "  == 1'b0)" << endl; 
	out << "    current_state <= ";	
	if(f.getFirstState()) {
	  bool notfound = true;
	  do {
	    if(resetstate == f.getStateID()) {
	      out << f.getStateName();
		notfound = false;
	    }
	  } while (f.getNextState());
	  if(notfound) {
	    if(f.getFirstState()) {
	      out << f.getStateName();
	    }
	  }
	}
	out << ";" << endl;   
	out << "  else " << endl;
	out << "    current_state <= next_state;" << endl;
	out << "end" << endl;


	
	//// Verification Exentions, can be used with Cadence formal verifier 

	if(Statetrans || Statecov || Transcov) {
	  
	  out << endl << endl << "// synopsys translate_off" << endl << endl;

	///// declare helper variables
	  
	  if(f.getFirstState()) {
	    do {
	      out << "wire state_" << f.getStateName() << ";" << endl;
	    } while (f.getNextState());
	  }
	  out << endl;
	  if(f.getFirstState()) {
	    do {
	      out << "assign state_" << f.getStateName() << " = (current_state == " << f.getStateName() << ");" << endl;
	    } while (f.getNextState());
	  }
	  out << endl;
	  
	//// Assertions for legal state transitions

	  if(Statetrans) {
	    if(f.getFirstState()) {
	      int stateid;
	      string statename;
	      do {
		stateid = f.getStateID();
		statename = f.getStateName();
		out << "a_legal_trans_"<< statename << " : assert property (@(posedge " << f.getClockName() << ") disable iff (!" << f.getResetName() << ") state_"<< statename << " |=> (";
		if(f.getFirstState()) {
		  int endstate;
		  bool firsthit = true;
		  bool reached = false;
		  do {
		    endstate = f.getStateID();
		    if(f.getFirstTrans()) {
		      do {
			if(f.getTransStart() == stateid) {
			  if(f.getTransEnd() == endstate) {
			    reached = true;
			  }
			}
		      } while (f.getNextTrans() && !reached);
		    }
		    if(reached) {
		      reached = false;
		      if(firsthit) {
			out << "state_"<< f.getStateName();
			firsthit = false;
		      } else {
			out << " || state_" << f.getStateName();
		      }
		    }	    
		  } while (f.getNextState());
		}
		
		out << "));" << endl;
		f.getStatebyID(stateid);
	      } while (f.getNextState());
	    }
	    out << endl;
	  }
	

      //// Transition coverage checking 

	  if(Transcov) {
	    if(f.getFirstState()) {
	      int stateid;
	      string statename;
	      do {
		stateid = f.getStateID();
		statename = f.getStateName();
		if(f.getFirstState()) {
		  int endstate;
		  bool exist = false;
		  do {
		    endstate = f.getStateID();
		    if(f.getFirstTrans()) {
		      do {
			if(f.getTransStart() == stateid) {
			  if(f.getTransEnd() == endstate) {
			    exist = true;
			  }
			}
		      } while (f.getNextTrans() && !exist);
		    }
		    if(exist) {
		      out << "c_trans_"<< statename << "_" << f.getStateName() << " : cover property (@(posedge " << f.getClockName() << ") disable iff (!" << f.getResetName() << ") state_"<< statename << " |=> state_" << f.getStateName() << ");" << endl;
		      
		      exist = false;
		    }		    
		  } while (f.getNextState());
		}
		f.getStatebyID(stateid);
	      } while (f.getNextState());
	    }
	    
	    out << endl;
	  }
	  
      //// State coverage checking
	  
	  if(Statecov) {
	    if(f.getFirstState()) {
	      do {
		out << "c_state_" << f.getStateName() << " : cover property (@(posedge "<< f.getClockName() <<") disable iff (!"<< f.getResetName() <<") " << f.getStateName() << ");" << endl;
	      } while (f.getNextState());
	    }
	    out << endl;
	  }
	  
	  out << endl << "// synopsys translate_on" << endl << endl;
	}
	
	out << endl << "endmodule" << endl;
	
	out.close();
	}
      }
    }
  }
}
