/***********************************************************************
*                                                                      *
* (C) 2007, Frank Lemke, Computer Architecture Group,                  *
* University of Mannheim, Germany                                      *
*                                                                      *
* This program is free software; you can redistribute it and/or modify *
* it under the terms of the GNU General Public License as published by *
* the Free Software Foundation; either version 2 of the License, or    *
* (at your option) any later version.                                  *
*                                                                      *
* This program is distributed in the hope that it will be useful,      *
* but WITHOUT ANY WARRANTY; without even the implied warranty of       *
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the        *
* GNU General Public License for more details.                         *
*                                                                      *
* You should have received a copy of the GNU General Public License    *
* along with this program; if not, write to the Free Software          *
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 *
* USA                                                                  *
*                                                                      *
* For informations regarding this file contact                         *
*			      office@mufasa.informatik.uni-mannheim.de *
*                                                                      *
***********************************************************************/

#ifndef ITEMS_H_
#define ITEMS_H_

#include <Qt3Support/Q3Canvas>
#include <Qt3Support/Q3CanvasEllipse>
#include <Qt3Support/Q3CanvasLine>
#include <Qt3Support/Q3CanvasRectangle>

class StateItem : public Q3CanvasEllipse {

 public:
  enum { RTTI = 1001 };

  int id;

  StateItem(int width, int height, Q3Canvas* can);
  ~StateItem();
  
  void drawShape(QPainter &painter);
  int rtti() const { return RTTI; }
  QString text();
  void setText(QString s);
  QRect boundingRect() const;
  void modifyPos(double posx, double posy);

 private:
  QString name;
};


class Transline : public Q3CanvasLine {
  
 public:
  enum { RTTI = 1002 };

  int id;

  Transline(Q3Canvas* can);
  ~Transline();

  void drawShape(QPainter &painter);
  int rtti() const { return RTTI; }
};

class Leveltransfer : public Q3CanvasEllipse{

 public:
  enum { RTTI = 1003 };

  int transid;
  int id;

  Leveltransfer(int width, int height, Q3Canvas* can);
  ~Leveltransfer();

  void drawShape(QPainter &painter);
  int rtti() const { return RTTI; }
  void modifyPos(double posx, double posy);
};

class LinkItem : public Q3CanvasEllipse{
  
 public:
  enum { RTTI = 1004 };

  int id;

  LinkItem(int width, int height, Q3Canvas* can);
  ~LinkItem();

  void drawShape(QPainter &painter);
  int rtti() const { return RTTI; }
  void modifyPos(double posx, double posy);
};

class Linkentrance : public Q3CanvasRectangle{

 public:
  enum { RTTI = 1005 };

  int transid;
  int id;

  Linkentrance(Q3Canvas* can);
  ~Linkentrance();
  
  void drawShape(QPainter &painter);
  int rtti() const { return RTTI; } 
  void modifyPos(double posx, double posy);
};

class Joinpoint : public Q3CanvasRectangle {

 public:
  enum { RTTI = 1006 };

  int transid;
  int id;

  Joinpoint(Q3Canvas* can);
  ~Joinpoint();

  void drawShape(QPainter &painter);
  int rtti() const { return RTTI; }
  void modifyPos(double posx, double posy);
};

class TrackpointItem : public Q3CanvasRectangle {
  
 public:
  enum { RTTI = 1007 };
  
  int transid;
  int id;
  
  TrackpointItem(Q3Canvas* can);
  ~TrackpointItem();
  
  void drawShape(QPainter &painter);
  int rtti() const { return RTTI; }
  void modifyPos(double posx, double posy);
};


class Hypertransition : public Q3CanvasEllipse {

 public:
  enum { RTTI = 1008 };

  int id;

  Hypertransition(int width, int height, Q3Canvas* can);
  ~Hypertransition();

  void drawShape(QPainter &painter);
  int rtti() const { return RTTI; } 
  void modifyPos(double posx, double posy);
};

class Hierarchyexit : public Q3CanvasRectangle{

 public:
  enum { RTTI = 1009 };

  int transid;
  int id;

  Hierarchyexit(Q3Canvas* can);
  ~Hierarchyexit();
  
  void drawShape(QPainter &painter);
  int rtti() const { return RTTI; } 
  void modifyPos(double posx, double posy);
};

class TranslineText : public Q3CanvasText {
  
 public:
  enum { RTTI = 1010 };

  int id;

  TranslineText(QString qs, Q3Canvas* can);
  ~TranslineText(){hide();};

  void draw(QPainter &painter);
  int rtti() const { return RTTI; }
  QString text();
  void setText(QString s);
  QRect boundingRect() const;
  void modifyPos(double posx, double posy);

 private:
  QString name;
};

#endif
